package iotcloud;

import java.util.List;
import java.util.ArrayList;

/**
 * Test cases.
 * @author Brian Demsky
 * @version 1.0
 */

public class Test {

    public static final  int NUMBER_OF_TESTS = 100;

    public static void main(String[] args)  throws ServerException {
        if (args[0].equals("2")) {
            test2();
        } else if (args[0].equals("3")) {
            test3();
        } else if (args[0].equals("4")) {
            test4();
        } else if (args[0].equals("5")) {
            test5();
        } else if (args[0].equals("6")) {
            test6();
        } else if (args[0].equals("7")) {
            test7();
        }
    }

    static void test7() throws ServerException {

        boolean foundError = false;
        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();

        // Make the Keys
        System.out.println("Setting up keys");
        for (int i = 0; i < 4; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }

        // Do Updates for the keys
        System.out.println("Setting Key-Values...");

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyB = "b" + i;
                String valueB = "b" + (i + t);

                IoTString iKeyB = new IoTString(keyB);
                IoTString iValueB = new IoTString(valueB);

                t1.startTransaction();
                t1.getSpeculativeAtomic(iKeyB);
                t1.addKV(iKeyB, iValueB);
                transStatusList.add(t1.commitTransaction());
            }
        }

        for (int i = 0; i < 4; i++) {

            String keyB = "b" + i;
            String valueB = "b" + (i + NUMBER_OF_TESTS - 1);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iValueB = new IoTString(valueB);

            IoTString testValB1 = t1.getSpeculative(iKeyB);
            IoTString testValB2 = t2.getSpeculative(iKeyB);

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB + "    " + testValB1);
                foundError = true;
            }

            if ((testValB2 != null)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }
        }

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyC = "c" + i;
                String valueC = "c" + (i + t);

                IoTString iKeyC = new IoTString(keyC);
                IoTString iValueC = new IoTString(valueC);

                t2.startTransaction();
                t2.getSpeculativeAtomic(iKeyC);
                t2.addKV(iKeyC, iValueC);
                transStatusList.add(t2.commitTransaction());
            }
        }

        for (int i = 0; i < 4; i++) {
            String keyC = "c" + i;
            String valueC = "c" + (i + NUMBER_OF_TESTS - 1);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iValueC = new IoTString(valueC);

            IoTString testValC1 = t1.getSpeculative(iKeyC);
            IoTString testValC2 = t2.getSpeculative(iKeyC);

            if ((testValC1 != null)) {
                System.out.println("Key-Value t1 incorrect: " + keyC + "   " + testValC1);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "   " + testValC2);
                foundError = true;
            }
        }

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyA = "a" + i;
                String keyD = "d" + i;
                String valueA = "a" + (i + t);
                String valueD = "d" + (i + t);

                IoTString iKeyA = new IoTString(keyA);
                IoTString iKeyD = new IoTString(keyD);
                IoTString iValueA = new IoTString(valueA);
                IoTString iValueD = new IoTString(valueD);

                t1.startTransaction();
                t1.addKV(iKeyA, iValueA);
                transStatusList.add(t1.commitTransaction());

                t2.startTransaction();
                t2.addKV(iKeyD, iValueD);
                transStatusList.add(t2.commitTransaction());
            }
        }

        System.out.println("Updating Clients...");
        t1.update();
        t2.update();
        t1.update();
        t2.update();

        System.out.println("Checking Key-Values...");
        for (int i = 0; i < 4; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + (i + NUMBER_OF_TESTS - 1);
            String valueB = "b" + (i + NUMBER_OF_TESTS - 1);
            String valueC = "c" + (i + NUMBER_OF_TESTS - 1);
            String valueD = "d" + (i + NUMBER_OF_TESTS - 1);

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA + "    " + testValA1);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB + "    " + testValB1);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC + "    " + testValC1);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD + "    " + testValD1);
                foundError = true;
            }

            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }

    static void test6() throws ServerException {

        boolean foundError = false;
        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();

        // Make the Keys
        System.out.println("Setting up keys");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }

        // Do Updates for the keys
        System.out.println("Setting Key-Values...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);

            t1.startTransaction();
            t1.getCommittedAtomic(iKeyA);
            t1.addKV(iKeyA, iValueA);
            transStatusList.add(t1.commitTransaction());

            t1.startTransaction();
            t1.getCommittedAtomic(iKeyB);
            t1.addKV(iKeyB, iValueB);
            transStatusList.add(t1.commitTransaction());

            t2.startTransaction();
            t2.getCommittedAtomic(iKeyC);
            t2.addKV(iKeyC, iValueC);
            transStatusList.add(t2.commitTransaction());

            t2.startTransaction();
            t2.getCommittedAtomic(iKeyD);
            t2.addKV(iKeyD, iValueD);
            transStatusList.add(t2.commitTransaction());
        }

        System.out.println("Updating Clients...");
        t1.update();
        t2.update();

        System.out.println("Checking Key-Values...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD);
                foundError = true;
            }


            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }

    static void test5() throws ServerException {

        boolean foundError = false;
        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();


        // Make the Keys
        System.out.println("Setting up keys");
        for (int i = 0; i < 4; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }

        // Do Updates for the keys
        System.out.println("Setting Key-Values...");

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyB = "b" + i;
                String valueB = "b" + (i + t);

                IoTString iKeyB = new IoTString(keyB);
                IoTString iValueB = new IoTString(valueB);

                t1.startTransaction();
                t1.addKV(iKeyB, iValueB);
                transStatusList.add(t1.commitTransaction());
            }
        }

        for (int i = 0; i < 4; i++) {

            String keyB = "b" + i;
            String valueB = "b" + (i + NUMBER_OF_TESTS - 1);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iValueB = new IoTString(valueB);

            IoTString testValB1 = t1.getSpeculative(iKeyB);
            IoTString testValB2 = t2.getSpeculative(iKeyB);

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB);
                foundError = true;
            }

            if ((testValB2 != null)) {
                System.out.println("Key-Value t2 incorrect: " + keyB);
                foundError = true;
            }
        }

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyC = "c" + i;
                String valueC = "c" + (i + t);

                IoTString iKeyC = new IoTString(keyC);
                IoTString iValueC = new IoTString(valueC);

                t2.startTransaction();
                t2.addKV(iKeyC, iValueC);
                transStatusList.add(t2.commitTransaction());
            }
        }

        for (int i = 0; i < 4; i++) {
            String keyC = "c" + i;
            String valueC = "c" + (i + NUMBER_OF_TESTS - 1);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iValueC = new IoTString(valueC);

            IoTString testValC1 = t1.getSpeculative(iKeyC);
            IoTString testValC2 = t2.getSpeculative(iKeyC);

            if ((testValC1 != null)) {
                System.out.println("Key-Value t1 incorrect: " + keyC + "   " + testValC1);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "   " + testValC2);
                foundError = true;
            }
        }

        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyA = "a" + i;
                String keyD = "d" + i;
                String valueA = "a" + (i + t);
                String valueD = "d" + (i + t);

                IoTString iKeyA = new IoTString(keyA);
                IoTString iKeyD = new IoTString(keyD);
                IoTString iValueA = new IoTString(valueA);
                IoTString iValueD = new IoTString(valueD);

                t1.startTransaction();
                t1.addKV(iKeyA, iValueA);
                transStatusList.add(t1.commitTransaction());

                t2.startTransaction();
                t2.addKV(iKeyD, iValueD);
                transStatusList.add(t2.commitTransaction());
            }
        }

        System.out.println("Updating Clients...");
        t1.update();
        t2.update();


        System.out.println("Checking Key-Values...");
        for (int i = 0; i < 4; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + (i + NUMBER_OF_TESTS - 1);
            String valueB = "b" + (i + NUMBER_OF_TESTS - 1);
            String valueC = "c" + (i + NUMBER_OF_TESTS - 1);
            String valueD = "d" + (i + NUMBER_OF_TESTS - 1);

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA + "    " + testValA1);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB + "    " + testValB1);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC + "    " + testValC1);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD + "    " + testValD1);
                foundError = true;
            }


            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }

    static void test4() throws ServerException {

        boolean foundError = false;
        long startTime = 0;
        long endTime = 0;
        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();

        // Make the Keys
        System.out.println("Setting up keys");
        startTime = System.currentTimeMillis();
        for (int i = 0; i < 4; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Key: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 4))   );
        System.out.println();


        // Do Updates for the keys
        System.out.println("Setting Key-Values...");
        startTime = System.currentTimeMillis();
        for (int t = 0; t < NUMBER_OF_TESTS; t++) {
            for (int i = 0; i < 4; i++) {
                String keyA = "a" + i;
                String keyB = "b" + i;
                String keyC = "c" + i;
                String keyD = "d" + i;
                String valueA = "a" + i;
                String valueB = "b" + i;
                String valueC = "c" + i;
                String valueD = "d" + i;

                IoTString iKeyA = new IoTString(keyA);
                IoTString iKeyB = new IoTString(keyB);
                IoTString iKeyC = new IoTString(keyC);
                IoTString iKeyD = new IoTString(keyD);
                IoTString iValueA = new IoTString(valueA);
                IoTString iValueB = new IoTString(valueB);
                IoTString iValueC = new IoTString(valueC);
                IoTString iValueD = new IoTString(valueD);

                t1.startTransaction();
                t1.addKV(iKeyA, iValueA);
                transStatusList.add(t1.commitTransaction());

                t1.startTransaction();
                t1.addKV(iKeyB, iValueB);
                transStatusList.add(t1.commitTransaction());

                t2.startTransaction();
                t2.addKV(iKeyC, iValueC);
                transStatusList.add(t2.commitTransaction());

                t2.startTransaction();
                t2.addKV(iKeyD, iValueD);
                transStatusList.add(t2.commitTransaction());
            }
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Update: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 16))   );
        System.out.println();


        System.out.println("Updating Clients...");
        t1.update();
        t2.update();


        System.out.println("Checking Key-Values...");
        for (int i = 0; i < 4; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD);
                foundError = true;
            }


            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }

    static void test3() throws ServerException {

        long startTime = 0;
        long endTime = 0;
        boolean foundError = false;

        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();


        // Make the Keys
        System.out.println("Setting up keys");
        startTime = System.currentTimeMillis();
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Key: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 4))   );
        System.out.println();


        // Do Updates for the keys
        System.out.println("Setting Key-Values...");
        startTime = System.currentTimeMillis();

        System.out.println("B...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String keyB = "b" + i;
            String valueB = "b" + i;

            IoTString iKeyB = new IoTString(keyB);
            IoTString iValueB = new IoTString(valueB);

            t1.startTransaction();
            t1.addKV(iKeyB, iValueB);
            transStatusList.add(t1.commitTransaction());
        }

        System.out.println("C...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String keyC = "c" + i;
            String valueC = "c" + i;

            IoTString iKeyC = new IoTString(keyC);
            IoTString iValueC = new IoTString(valueC);

            t2.startTransaction();
            t2.addKV(iKeyC, iValueC);
            transStatusList.add(t2.commitTransaction());
        }

        System.out.println("A, D...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String keyA = "a" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueD = new IoTString(valueD);

            t1.startTransaction();
            t1.addKV(iKeyA, iValueA);
            transStatusList.add(t1.commitTransaction());

            t2.startTransaction();
            t2.addKV(iKeyD, iValueD);
            transStatusList.add(t2.commitTransaction());
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Update: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 4))   );
        System.out.println();


        System.out.println("Updating Clients...");
        t1.update();
        t2.update();


        System.out.println("Checking Key-Values...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD);
                foundError = true;
            }


            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }

    static void test2() throws ServerException {

        boolean foundError = false;
        long startTime = 0;
        long endTime = 0;
        List<TransactionStatus> transStatusList = new ArrayList<TransactionStatus>();

        // Setup the 2 clients
        Table t1 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 321);
        t1.initTable();
        System.out.println("T1 Ready");

        Table t2 = new Table("http://127.0.0.1/test.iotcloud/", "reallysecret", 351);
        t2.update();
        System.out.println("T2 Ready");

        // Make the Keys
        System.out.println("Setting up keys");
        startTime = System.currentTimeMillis();
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String a = "a" + i;
            String b = "b" + i;
            String c = "c" + i;
            String d = "d" + i;
            IoTString ia = new IoTString(a);
            IoTString ib = new IoTString(b);
            IoTString ic = new IoTString(c);
            IoTString id = new IoTString(d);
            t1.createNewKey(ia, 321);
            t1.createNewKey(ib, 351);
            t2.createNewKey(ic, 321);
            t2.createNewKey(id, 351);
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Key: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 4))   );
        System.out.println();

        // Do Updates for the keys
        System.out.println("Setting Key-Values...");
        startTime = System.currentTimeMillis();
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {
            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);

            t1.startTransaction();
            t1.addKV(iKeyA, iValueA);
            transStatusList.add(t1.commitTransaction());


            t1.startTransaction();
            t1.addKV(iKeyB, iValueB);
            transStatusList.add(t1.commitTransaction());


            t2.startTransaction();
            t2.addKV(iKeyC, iValueC);
            transStatusList.add(t2.commitTransaction());


            t2.startTransaction();
            t2.addKV(iKeyD, iValueD);
            transStatusList.add(t2.commitTransaction());
        }
        endTime = System.currentTimeMillis();
        System.out.println("Time Taken: " + (double)   ((endTime - startTime) / 1000.0)    );
        System.out.println("Time Taken Per Update: " + (double)  (((endTime - startTime) / 1000.0) / (NUMBER_OF_TESTS * 4))   );
        System.out.println();


        System.out.println("Updating Clients...");
        t1.update();
        t2.update();


        System.out.println("Checking Key-Values...");
        for (int i = 0; i < NUMBER_OF_TESTS; i++) {

            String keyA = "a" + i;
            String keyB = "b" + i;
            String keyC = "c" + i;
            String keyD = "d" + i;
            String valueA = "a" + i;
            String valueB = "b" + i;
            String valueC = "c" + i;
            String valueD = "d" + i;

            IoTString iKeyA = new IoTString(keyA);
            IoTString iKeyB = new IoTString(keyB);
            IoTString iKeyC = new IoTString(keyC);
            IoTString iKeyD = new IoTString(keyD);
            IoTString iValueA = new IoTString(valueA);
            IoTString iValueB = new IoTString(valueB);
            IoTString iValueC = new IoTString(valueC);
            IoTString iValueD = new IoTString(valueD);


            IoTString testValA1 = t1.getCommitted(iKeyA);
            IoTString testValB1 = t1.getCommitted(iKeyB);
            IoTString testValC1 = t1.getCommitted(iKeyC);
            IoTString testValD1 = t1.getCommitted(iKeyD);

            IoTString testValA2 = t2.getCommitted(iKeyA);
            IoTString testValB2 = t2.getCommitted(iKeyB);
            IoTString testValC2 = t2.getCommitted(iKeyC);
            IoTString testValD2 = t2.getCommitted(iKeyD);

            if ((testValA1 == null) || (testValA1.equals(iValueA) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyA);
                foundError = true;
            }

            if ((testValB1 == null) || (testValB1.equals(iValueB) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyB);
                foundError = true;
            }

            if ((testValC1 == null) || (testValC1.equals(iValueC) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyC);
                foundError = true;
            }

            if ((testValD1 == null) || (testValD1.equals(iValueD) == false)) {
                System.out.println("Key-Value t1 incorrect: " + keyD);
                foundError = true;
            }


            if ((testValA2 == null) || (testValA2.equals(iValueA) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyA + "    " + testValA2);
                foundError = true;
            }

            if ((testValB2 == null) || (testValB2.equals(iValueB) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyB + "    " + testValB2);
                foundError = true;
            }

            if ((testValC2 == null) || (testValC2.equals(iValueC) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyC + "    " + testValC2);
                foundError = true;
            }

            if ((testValD2 == null) || (testValD2.equals(iValueD) == false)) {
                System.out.println("Key-Value t2 incorrect: " + keyD + "    " + testValD2);
                foundError = true;
            }
        }

        for (TransactionStatus status : transStatusList) {
            if (status.getStatus() != TransactionStatus.StatusCommitted) {
                foundError = true;
                System.out.println(status.getStatus());
            }
        }

        if (foundError) {
            System.out.println("Found Errors...");
        } else {
            System.out.println("No Errors Found...");
        }
    }
}
